@@ -2,7 +2,7 @@ calc.area <- function(coords, gaps){
 #Function to calculate the are of a study region
   list.area <- function(coords.list){
     matrix.coords <- lapply(coords.list, as.matrix)
-    areas <- lapply(matrix.coords, areapl)
+    areas <- lapply(matrix.coords, splancs::areapl)
     areas <- unlist(areas)
     total.area <- sum(areas) 
     return(total.area)
